Package: OhdsiReportGenerator
Type: Package
Title: Observational Health Data Sciences and Informatics Report
        Generator
Version: 2.2.0
Date: 2026-05-15
Authors@R: c(
  person("Jenna", "Reps", email = "jreps@its.jnj.com", role = c("aut", "cre")),
  person("Anthony", "Sena", email = "sena@ohdsi.org", role = c("aut"))
  )
Maintainer: Jenna Reps <jreps@its.jnj.com>
Description: Extract results into R from the Observational Health Data Sciences and Informatics result database (see <https://ohdsi.github.io/Strategus/results-schema/index.html>) and generate reports/presentations via 'quarto' that summarize results in HTML format. Learn more about 'OhdsiReportGenerator' at <https://ohdsi.github.io/OhdsiReportGenerator/>.
License: Apache License 2.0
URL: https://ohdsi.github.io/OhdsiReportGenerator/,
        https://github.com/OHDSI/OhdsiReportGenerator
BugReports: https://github.com/OHDSI/OhdsiReportGenerator/issues
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: CirceR, DatabaseConnector, forestploter, dplyr, ggplot2,
        grDevices, grid, gt, gtExtras, kableExtra, ParallelLogger,
        quarto, rlang, rmarkdown, SqlRender, tidyr
Suggests: htmltools, knitr, markdown, ResultModelManager, RSQLite,
        testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-16 02:02:23 UTC; jreps
Author: Jenna Reps [aut, cre],
  Anthony Sena [aut]
Repository: CRAN
Date/Publication: 2026-05-16 23:10:02 UTC
Built: R 4.6.0; ; 2026-05-17 01:28:48 UTC; unix
