# Generated by roxygen2: do not edit by hand

export(addTarColumn)
export(createCharacterizationIndexes)
export(createCohortIndexes)
export(createIncidenceIndexes)
export(createPredictionReport)
export(createSccsIndexes)
export(formatBinaryCovariateName)
export(generateFullReport)
export(generateSummaryPredictionReport)
export(getAnalysisCohorts)
export(getBinaryCaseSeries)
export(getBinaryRiskFactors)
export(getBinaryTargetBaseline)
export(getCMEstimation)
export(getCaseCounts)
export(getCaseTargetCounts)
export(getCharacterizationCohortBinary)
export(getCharacterizationCohortContinuous)
export(getCharacterizationDemographics)
export(getCharacterizationOutcomes)
export(getCharacterizationTargets)
export(getCmDiagnosticsData)
export(getCmMetaEstimation)
export(getCmNegativeControlEstimates)
export(getCmOutcomes)
export(getCmPropensityModel)
export(getCmTable)
export(getCmTargets)
export(getCohortAttrition)
export(getCohortCounts)
export(getCohortDefinitions)
export(getCohortInclusionRules)
export(getCohortInclusionStats)
export(getCohortInclusionSummary)
export(getCohortMeta)
export(getCohortSubsetAttrition)
export(getCohortSubsetDefinitions)
export(getContinuousCaseSeries)
export(getContinuousRiskFactors)
export(getDatabaseDetails)
export(getDechallengeRechallenge)
export(getDechallengeRechallengeFails)
export(getEventDuration)
export(getExampleConnectionDetails)
export(getFullPredictionPerformances)
export(getIncidenceOutcomes)
export(getIncidenceRates)
export(getIncidenceTargets)
export(getOutcomeTable)
export(getPredictionAggregateTopPredictors)
export(getPredictionCohorts)
export(getPredictionCovariates)
export(getPredictionDiagnosticTable)
export(getPredictionDiagnostics)
export(getPredictionHyperParamSearch)
export(getPredictionIntercept)
export(getPredictionLift)
export(getPredictionModelDesigns)
export(getPredictionOutcomes)
export(getPredictionPerformanceTable)
export(getPredictionPerformances)
export(getPredictionTargets)
export(getPredictionTopPredictors)
export(getSccsDiagnosticsData)
export(getSccsEstimation)
export(getSccsMetaEstimation)
export(getSccsModel)
export(getSccsNegativeControlEstimates)
export(getSccsOutcomes)
export(getSccsTable)
export(getSccsTargets)
export(getSccsTimeToEvent)
export(getSubsetText)
export(getTargetBinaryFeatures)
export(getTargetContinuousFeatures)
export(getTargetTable)
export(getTimeToEvent)
export(getTreatmentPathways)
export(kableDark)
export(plotAgeDistributions)
export(plotCmEstimates)
export(plotSccsEstimates)
export(plotSexDistributions)
export(processCohortDefinitionsForQuarto)
export(processCohorts)
export(removeSpaces)
export(restrictCohortDefinitionsForQuarto)
export(viewIncidenceRate)
importFrom(dplyr,"%>%")
importFrom(rlang,":=")
importFrom(rlang,.data)
