# Causal Analyses {.break-slides}

## Risk Estimation Methods

::: {style="font-size: 50%;"}

### Comparative Cohort Design (Cohort Method)
Comparison of outcomes between two or more treatment cohorts, while controlling for other factors that may influence the results (here: via propensity score matching).

![Illustration of CohortMethod Design](pictures/cohort_method.png)

### Self-Controlled Case Series (SCCS)
Comparison of outcome incidence between time periods at risk (here: based on treatment exposure) and non-exposed periods; individuals serve as their own controls. 

![Illustration of SCCS design](pictures/sccs.png)

:::