## Comments

::: {style="font-size: 70%;"}

- There were several potential limitations to this analysis, which concern the provenance, purpose, and use of the very diverse datasets considered for this analysis:

- Our analysis observed data from those subjects who presented for medical attention; those who did not seek medical attention could not be included and may affect the metrics in this study. Those with milder disease may also not be presented or be underrepresented.

- There might be coding inaccuracies in the recording of diagnose and procedures, as the data were originally not collected for research purposes; the databases might, e.g., not accurately capture all events and event dates.

- Furthermore, findings in uninsured populations, populations with restricted healthcare access, or other countries and healthcare systems may differ from the findings in this project. 

:::