## {{title}}

:::: {.columns}
::: {.column width="70%"}

:::: {style="font-size: 50%;"}

### Background: 
{{trigger}}

### General safety question
{{safetyQuestion}}

### ASSURE request/objective: 
{{objective}}

:::: 

:::

::: {.column width="30%"}

:::: {style="font-size: 50%;"}
- Assure lead: {{lead}}
- Assure team: {{team}}
- Date: {{date}}
::::

:::

::::

::: footer

Analysis information and ASSURE team details.

:::




