---
  output: html_document
---

```{r results='asis', echo=FALSE}
cat('\n ',paste0('Validation on ', result$database), '\n')
```

#### Attrition

```{r results='asis', echo=FALSE}
data <- result$attrition %>%
  dplyr::select(-"performanceId")
  
  kableDark(data = data)
```

#### Summary

```{r results='asis', echo=FALSE}
data <- result$evaluation_statistics %>%
  dplyr::select(-"performanceId") %>%
  tidyr::pivot_wider(
    id_cols =  "metric",  #.data$metric, 
    names_from = "evaluation", #.data$evaluation, 
    values_from = "value" #.data$value
    ) 
kableDark(data = data)
```

#### Plots

add roc and smooth cal plots here
  
#### Covariates

```{r results='asis', echo=FALSE}
data <- result$covariate_summary %>%
  dplyr::select(
    c(
    "covariateName", 
    "covariateValue",
    "covariateCount",
    "withNoOutcomeCovariateMean",
    "withOutcomeCovariateMean",
    "standardizedMeanDiff"
    )
    )
kableDark(data = data)
```
  