## Signal Evaluation Overview

```{r}
#| label: overview_sub
#| echo: false
#| results: 'asis'

cat('\n\n:::: {.columns}')
cat('\n\n::: {.column width="50%"}')
cat(paste0('\n\n> Target: ', targetName,' @sec-target'))

if(!is.null(comparators)){
cat(paste0('\n\n> Comparators: ',paste0(comparators$cohortFriendlyName, ' @sec-',OhdsiReportGenerator::removeSpaces(comparators$cohortFriendlyName), collapse =', ')))
}

if(!is.null(indications)){
cat(paste0('\n\n> Indication/s: ', paste0(indications$cohortFriendlyName,' @sec-',OhdsiReportGenerator::removeSpaces(indications$cohortFriendlyName), collapse =', ')))
}

cat(paste0('\n\n> Outcome/s: ', paste0(outcomeNames, ' @sec-',OhdsiReportGenerator::removeSpaces(outcomeNames), collapse =', ')))
cat('\n\n:::')

cat('\n\n::: {.column width="10%"}')
cat('\n\n<!-- empty column to create gap -->')
cat('\n\n:::')

cat('\n\n::: {.column width="40%"}')
cat('\n\n::: {.border}')
cat('\n\n::: {style="font-size: 50%;"}')

cat('\n\n| Study Period: ')
cat(paste0('\n| ', studyPeriod))

cat('\n\n| Restrictions:') 
cat(paste0('\n| ', restrictions))

cat('\n\n| Analytic Methods:') 
if(params$includeCharacterization | params$includeCI){
cat('\n|   * Characterization')
}
if(params$includeCM){
cat('\n|   * Propensity-matched comparative cohort on treatment')
}
if(params$includeSCCS){
cat('\n|   * Self-controlled case series (SCCS)')
}
if(params$includePLP){
cat('\n|   * Patient-level prediction')
}

# TODO - update this based on databases in results
cat('\n\n| Databases: ')
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['us claims']]),0,dbs[['us claims']]), ' US Claims'))
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['us ehr']]),0,dbs[['us ehr']]), '  US EHR'))
cat(paste0('\n|   *  ', ifelse(is.null(dbs[['non-us claims']]),0,dbs[['non-us claims']]), '  Non-US Claims'))

cat('\n\n:::')
cat('\n\n:::')
cat('\n\n:::')

cat('\n\n::::')

```
